import tkinter as tk
from tkinter import *
import PIL.Image
from PIL import Image, ImageTk
import os
from os import listdir
from os.path import isfile, join
import time, random
from datetime import datetime
# import datetime
import RPi.GPIO as GPIO 
import Adafruit_DHT
from bmp280 import BMP280

del_sw = 38
shdn_sw = 40
button_state = False
shdn = False

# DHT22 sensor port (PIN 4)
# I2C address 76
DHT_SENSOR = Adafruit_DHT.DHT22
DHT_PIN = 4

# define BMP280 sensor definition & initialization
try:
    from smbus2 import SMBus
except ImportError:
    from smbus import SMBus
bus = SMBus(1)
bmp280 = BMP280(i2c_dev=bus)

def button_callback(channel):
	global button_state

	button_state = True
	print("Button was pushed! "+str(button_state))

def button_shutdown(channel):
	global shdn

	shdn = True
	print("Shutdown started....")

class ImageViewer():
	def __init__(self):
		self.window = tk.Tk()
#		self.window.title("Image Viewer")
		self.window.overrideredirect(1)
		self.window.config(cursor='none')
		self.window.geometry("800x480+0+0")

		self.load_images()
		self.canvas = tk.Canvas(self.window, width=800, height=480, bd=0, highlightthickness=0, relief='ridge')
		self.canvas.grid(row=0, column=0)
		self.img = None
		self.image_on_canvas = self.canvas.create_image(0, 0, anchor=tk.NW)
		self.image_idx = random.randint(0, 730)
		self.update_screen()
		self.window.mainloop()

	def load_images(self):
		self.mypath = '/home/pi/IMAGE/'
		self.imgfiles = [f for f in listdir(self.mypath) if isfile(join(self.mypath, f))]

	def update_screen(self):
		global button_state
		global shdn

		while True:
			self.image_idx = random.randint(0, len(self.imgfiles))
# Try catch를 써야함
# randome이 이미 지워진 index를 언급하는 경우에는 에러 발생
			try:
				if os.path.exists(self.mypath+self.imgfiles[self.image_idx]):
					break
			except:
				continue
		self.im = PIL.Image.open(self.mypath+self.imgfiles[self.image_idx])
		self.basewidth = 800
		self.wpercent = (self.basewidth / float(self.im.size[0]))
		self.hsize = int((float(self.im.size[1]) * float(self.wpercent)))
		self.im = self.im.resize((self.basewidth, self.hsize), Image.ANTIALIAS)
		self.img = PIL.ImageTk.PhotoImage(self.im)
		self.canvas.itemconfig(self.image_on_canvas, image=self.img)
		self.show_time()
		self.pushed = False
		for i in range(0, 6):
			self.canvas.update()
			if button_state:
				print("Checked....")
				self.canvas.create_text(0, 0, text=self.imgfiles[self.image_idx], font=("Arial bold", 40), fill="red", anchor=tk.NW, tag="del_msg")
				self.canvas.update()
				button_state = False
				time.sleep(0.5)
				self.pushed = True
			elif shdn:
				os.system('sync')
				time.sleep(1)
				os.system('sudo shutdown -h 0')
			time.sleep(0.5)
		if self.pushed:
			button_state = False
			time.sleep(0.5)
			self.canvas.delete("del_msg")
			self.delete_file()
		self.window.after(3000, self.update_screen)

	def delete_file(self):
		if os.path.exists(self.mypath+self.imgfiles[self.image_idx]):
			f = open('deleted_file_list.txt', 'a')
			f.write(self.imgfiles[self.image_idx]+'\n')
			f.close()
			cmd = 'mv '+self.mypath+self.imgfiles[self.image_idx]+' /home/pi/DELETED/'
			os.system(cmd)

	def show_time(self):
		global DHT_SENSOR
		global DHT_PIN
		global bmp280

		self.humidity = None
		self.temperature = None
		self.pressure = None

		self.hum_form = '{0:.1f}'
		self.tmp_form = '{0:.1f}'
		self.pre_form = '{:05.1f}'

		self.dayofweek = ['MON', 'TUE', 'WED', 'THD', 'FRI', 'SAT', 'SUN']

# open logging file
		self.log_path = '/home/pi/viewer/log/'
		self.file_name = self.log_path+datetime.now().strftime("%Y%m%d")+'.txt'
		if os.path.exists(self.file_name):
# if file exist, open it as append mode
			g = open(self.file_name, '+a')
		else:
			g = open(self.file_name, 'w')

		while self.humidity == None and self.temperature == None:
			self.humidity, self.temperature = Adafruit_DHT.read_retry(DHT_SENSOR, DHT_PIN)
			time.sleep(0.5)
		self.hum = self.hum_form.format(self.humidity)
		self.temp = self.tmp_form.format(self.temperature)

		while self.pressure == None:
			self.pressure = bmp280.get_pressure()
			time.sleep(0.5)
		self.press = self.pre_form.format(self.pressure)

		self.canvas.delete("time_str")
		self.idx = datetime.today().weekday()
		self.tm_stamp = datetime.now().strftime("%Y-%m-%d %H:%M:%S")+' '+self.dayofweek[self.idx]
		self.tm_stamp += ' '+str(self.hum)+'%'+' '+str(self.temp)+'C'+' '+str(self.press)+'hPa'
		self.canvas.create_text(55, 440, text=self.tm_stamp, font=("Arial bold", 20), fill="red", anchor=tk.NW, tag="time_str")
		g.write(self.tm_stamp+'\n')
		g.close()

GPIO.setwarnings(False) # Ignore warning for now
GPIO.setmode(GPIO.BOARD) # Use physical pin numbering
GPIO.setup(del_sw, GPIO.IN, pull_up_down=GPIO.PUD_UP) # GPIO 38
GPIO.setup(shdn_sw, GPIO.IN, pull_up_down=GPIO.PUD_UP) # GPIO 40

GPIO.add_event_detect(del_sw, GPIO.RISING, callback=button_callback, bouncetime=300) # GPIO 38
GPIO.add_event_detect(shdn_sw, GPIO.RISING, callback=button_shutdown, bouncetime=300) # GPIO 40
# shdn = False

app = ImageViewer()

GPIO.cleanup() # Clean up
